/*
 * setup-softdevice.c
 *
 * See the README file for copyright information and how to reach the authors.
 *
 * $Id: setup-softdevice.c,v 1.2 2004/07/10 20:57:01 stefan Exp $
 */

#include "setup-softdevice.h"

/* ----------------------------------------------------------------------------
 * index to this array correspond to AFD values
 */
char *crop_str [] = {
        "none",
        "4:3",
        "16:9",
        NULL
     };

char *deint_str [] = {
        "none",
        "lavc",
#ifdef FB_SUPPORT
        "FB-intern",
#endif //FB_SUPPORT
        NULL
     };

/* ---------------------------------------------------------------------------
 */

cSetupStore setupStore;

cSetupStore::cSetupStore ()
{
#ifdef XV_SUPPORT
  outputMethod = 0;
  xvAspect     = XV_FORMAT_NORMAL;
#endif //XV_SUPPORT
  cropMode   = 0;
  deintMethod  = 0;
}

bool cSetupStore::SetupParse(const char *Name, const char *Value)
{
  if (!strcasecmp(Name,"Deinterlace Method")) {
    deintMethod = atoi(Value);
    if (deintMethod < 0 | deintMethod > 1) deintMethod = 0;
    fprintf (stderr,
            "[softdevice] deinterlace method set to %s\n",
            deint_str [deintMethod]);
  } else if(!strcasecmp(Name,"CropMode")) {
    cropMode = atoi(Value);
    fprintf (stderr, "[softdevice] cropping mode set to %d (%s)\n",
             cropMode,
             (cropMode < 0) ? "Invalid":
             (cropMode > 2) ? "invalid":
             crop_str [cropMode]);
#ifdef XV_SUPPORT
  } else if(!strcasecmp(Name, "Xv-Aspect")) {
    xvAspect = atoi(Value);
#endif //XV_SUPPORT
  } else return false;

  return true;
}


/* ---------------------------------------------------------------------------
 */
cMenuSetupSoftdevice::cMenuSetupSoftdevice(void)
{
  data = setupStore;
#ifdef XV_SUPPORT
  Add(new cMenuEditIntItem(tr("Xv-Aspect"),
                           &data.xvAspect));
#endif //XV_SUPPORT
  Add(new cMenuEditStraItem(tr("CropMode"),
                            &data.cropMode,
                            3,
                            crop_str));
  Add(new cMenuEditStraItem(tr("Deinterlace Method"),
                            &data.deintMethod,
#ifdef FB_SUPPORT
                            3,
#else
                            2,
#endif //FB_SUPPORT
                            deint_str));
}


/* ---------------------------------------------------------------------------
 */
void cMenuSetupSoftdevice::Store(void)
{
  if (setupStore.deintMethod != data.deintMethod) {
    printf("[softdevice] deinterlace method changed to %s\n", deint_str [data.deintMethod]);
  }

  setupStore = data;
#ifdef XV_SUPPORT
  SetupStore ("Xv-Aspect",          setupStore.xvAspect);
#endif //XV_SUPPORT
  SetupStore ("CropMode",           setupStore.cropMode);
  SetupStore ("Deinterlace Method", setupStore.deintMethod);
}
